import torch
import torch.nn as nn
from torch.nn.utils import spectral_norm



class ResNet_Head(nn.Module):
    def __init__(
            self,
            feature_extractor,
            num_classes,
            centroid_size,
            model_output_size,
            length_scale,
            gamma,
    ):
        super().__init__()
        self.feature_extractor = feature_extractor
        self.head = nn.Sequential(
            nn.ReLU(),
            nn.Linear(model_output_size, num_classes),
            nn.Softmax(dim=1)
        )

    def forward(self, x):
        x = self.feature_extractor(x)
        x = self.head(x)
        return x

    def update_embeddings(self, x, y):
        pass


class ResNet_DUQ(nn.Module):
    def __init__(
        self,
        feature_extractor,
        num_classes,
        centroid_size,
        model_output_size,
        length_scale,
        gamma,
    ):
        super().__init__()

        self.gamma = gamma

        self.W = nn.Parameter(
            torch.zeros(centroid_size, num_classes, model_output_size)
        )
        nn.init.kaiming_normal_(self.W, nonlinearity="relu")

        self.feature_extractor = feature_extractor

        self.register_buffer("N", torch.zeros(num_classes) + 13)
        self.register_buffer(
            "m", torch.normal(torch.zeros(centroid_size, num_classes), 0.05)
        )
        self.m = self.m * self.N

        self.sigma = length_scale

    def rbf(self, z):
        z = torch.einsum("ij,mnj->imn", z, self.W)

        embeddings = self.m / self.N.unsqueeze(0)
        self.embeddings = embeddings

        diff = z - embeddings.unsqueeze(0)
        diff = (diff ** 2).mean(1).div(2 * self.sigma ** 2).mul(-1).exp()

        return diff

    def update_embeddings(self, x, y):
        self.N = self.gamma * self.N + (1 - self.gamma) * y.sum(0)

        z = self.feature_extractor(x)

        z = torch.einsum("ij,mnj->imn", z, self.W)
        embedding_sum = torch.einsum("ijk,ik->jk", z, y)

        self.m = self.gamma * self.m + (1 - self.gamma) * embedding_sum

    def forward(self, x):
        z = self.feature_extractor(x)
        self.z = z
        y_pred = self.rbf(z)

        return y_pred


class ResNet_DUQ2(nn.Module):
    def __init__(
        self,
        feature_extractor,
        num_classes,
        centroid_size,
        model_output_size,
        length_scale,
        gamma,
    ):
        super().__init__()
        self.gamma = gamma

        self.feature_extractor = feature_extractor
        self.sigma = length_scale

    def rbf(self, z):
        pass

    def update_embeddings(self, x, y):
        pass

    def forward(self, x):
        z = self.feature_extractor(x)
        self.z = z
        y_pred = self.rbf(z)

        return y_pred
